##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""Hotfix 2001-03-08"""


# A workaround for the broken aq_inContextOf
from Acquisition import aq_base, aq_inner, aq_parent

def isInContext(fixed, variable):
    fixed = aq_base(fixed)
    while variable is not None:
        v = aq_base(variable)
        if v is fixed:
            return 1
        variable = aq_parent(aq_inner(variable))
    return 0


if 1:
    def allowed(self, object, object_roles=None):
        """Check whether the user has access to object, assuming that
           object.__roles__ is the given roles."""
        if object_roles is None or 'Anonymous' in object_roles:
            return 1
        usr_roles=self.getRolesInContext(object)
        for role in object_roles:
            if role in usr_roles:
                # The user apparently has one of the necessary
                # roles, but first make sure the object exists
                # in the context of the parent of the acl_users
                # folder.
                ufolder = getattr(self, 'aq_parent', None)
                ucontext = getattr(ufolder, 'aq_parent', None)
                if ucontext is not None:
                    if object is None:
                        # This is a strange rule, though
                        # it doesn't cause any security holes. SDH
                        return 1
                    if not hasattr(object, 'aq_inContextOf'):
                        if hasattr(object, 'im_self'):
                            # This is a method.  Grab its self.
                            object=object.im_self
                        if not hasattr(object, 'aq_inContextOf'):
                            # object is not wrapped, therefore we
                            # can't determine context.
                            # Fail the access attempt.  Otherwise
                            # this would be a security hole.
                            return None
                    # XXX this is a workaround for broken aq_inContextOf
                    # if not object.aq_inContextOf(ucontext, 1):
                    if not isInContext(ucontext, object):
                        if 'Shared' in object_roles:
                            # Damn, old role setting. Waaa
                            object_roles=self._shared_roles(object)
                            if 'Anonymous' in object_roles: return 1
                        return None
                # Note that if self were not wrapped, it would
                # not be possible to determine the user's context
                # and this method would return 1.
                # However, as long as user folders always return
                # wrapped user objects, this is safe.
                return 1


# install the fixes
import AccessControl.User
AccessControl.User.isInContext = isInContext
AccessControl.User.BasicUser.allowed = allowed
